import os
import unittest
from pyats.topology import loader
from genie.libs.sdk.apis.iosxe.bgp.configure import get_running_config_section_dict


class TestGetRunningConfigSectionDict(unittest.TestCase):

    @classmethod
    def setUpClass(self):
        testbed = f"""
        devices:
          iolpe2:
            connections:
              defaults:
                class: unicon.Unicon
              a:
                command: mock_device_cli --os iosxe --mock_data_dir {os.path.dirname(__file__)}/mock_data --state connect
                protocol: unknown
            os: iosxe
            platform: iol
            type: iol
        """
        self.testbed = loader.load(testbed)
        self.device = self.testbed.devices['iolpe2']
        self.device.connect(
            learn_hostname=True,
            init_config_commands=[],
            init_exec_commands=[]
        )

    def test_get_running_config_section_dict(self):
        result = get_running_config_section_dict(self.device)
        expected_output = {'Building configuration...': {},
 'Current configuration : 8337 bytes': {},
 'boot-end-marker': {},
 'boot-start-marker': {},
 'clock timezone PST -8 0': {},
 'control-plane': {},
 'end': {},
 'hostname iolpe2': {},
 'interface Ethernet0/0': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet0/1': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet0/2': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet0/3': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet1/0': {'description to iolce1 Ethernet2/0': {},
                           'ip address 99.1.3.3 255.255.255.0': {},
                           'ipv6 address 2001::99:1:3:3/112': {},
                           'vrf forwarding ce1': {}},
 'interface Ethernet1/1': {'description to iolce1 Ethernet2/1': {},
                           'ip address 100.1.3.3 255.255.255.0': {},
                           'ipv6 address 2001::100:1:3:3/112': {},
                           'vrf forwarding ce1': {}},
 'interface Ethernet1/2': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet1/3': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet2/0': {'description to iolpe1 Ethernet2/0': {},
                           'ip address 99.2.3.3 255.255.255.0': {},
                           'ip router isis sr': {},
                           'ip rsvp bandwidth': {},
                           'ipv6 address 2001::99:2:3:3/112': {},
                           'ipv6 router isis sr': {},
                           'isis network point-to-point': {},
                           'mpls ip': {},
                           'mpls traffic-eng tunnels': {}},
 'interface Ethernet2/1': {'description to iolpe1 Ethernet2/1': {},
                           'ip address 100.2.3.3 255.255.255.0': {},
                           'ip router isis sr': {},
                           'ip rsvp bandwidth': {},
                           'ipv6 address 2001::100:2:3:3/112': {},
                           'ipv6 router isis sr': {},
                           'isis network point-to-point': {},
                           'mpls ip': {},
                           'mpls traffic-eng tunnels': {}},
 'interface Ethernet2/2': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet2/3': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet3/0': {'description to iolp2 Ethernet1/0': {},
                           'ip address 99.3.5.3 255.255.255.0': {},
                           'ip router isis sr': {},
                           'ip rsvp bandwidth': {},
                           'ipv6 address 2001::99:3:5:3/112': {},
                           'ipv6 router isis sr': {},
                           'isis network point-to-point': {},
                           'mpls ip': {},
                           'mpls traffic-eng tunnels': {}},
 'interface Ethernet3/1': {'description to iolp2 Ethernet1/1': {},
                           'ip address 100.3.5.3 255.255.255.0': {},
                           'ip router isis sr': {},
                           'ip rsvp bandwidth': {},
                           'ipv6 address 2001::100:3:5:3/112': {},
                           'ipv6 router isis sr': {},
                           'isis network point-to-point': {},
                           'mpls ip': {},
                           'mpls traffic-eng tunnels': {}},
 'interface Ethernet3/2': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet3/3': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet4/0': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet4/1': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet4/2': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet4/3': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet5/0': {'description to iolp1 Ethernet5/0': {},
                           'ip address 99.3.4.3 255.255.255.0': {},
                           'ip router isis sr': {},
                           'ip rsvp bandwidth': {},
                           'ipv6 address 2001::99:3:4:3/112': {},
                           'ipv6 router isis sr': {},
                           'isis network point-to-point': {},
                           'mpls ip': {},
                           'mpls traffic-eng tunnels': {}},
 'interface Ethernet5/1': {'description to iolp1 Ethernet5/1': {},
                           'ip address 100.3.4.3 255.255.255.0': {},
                           'ip router isis sr': {},
                           'ip rsvp bandwidth': {},
                           'ipv6 address 2001::100:3:4:3/112': {},
                           'ipv6 router isis sr': {},
                           'isis network point-to-point': {},
                           'mpls ip': {},
                           'mpls traffic-eng tunnels': {}},
 'interface Ethernet5/2': {'no ip address': {}, 'shutdown': {}},
 'interface Ethernet5/3': {'no ip address': {}, 'shutdown': {}},
 'interface Loopback1': {'ip address 1.1.1.3 255.255.255.255': {},
                         'ip router isis sr': {},
                         'ipv6 address 2001::1:1:1:3/128': {},
                         'ipv6 router isis sr': {}},
 'ip cef': {},
 'ip forward-protocol nd': {},
 'ip http secure-server': {},
 'ip http server': {},
 'ip ssh bulk-mode 131072': {},
 'ipv6 cef': {},
 'ipv6 unicast-routing': {},
 'line aux 0': {},
 'line con 0': {'exec-timeout 0 0': {}, 'logging synchronous': {}},
 'line vty 0 4': {'exec-timeout 0 0': {},
                  'login': {},
                  'transport input ssh': {}},
 'logging buffered 4000000': {},
 'login on-success log': {},
 'memory free low-watermark processor 54960': {},
 'mpls traffic-eng tunnels': {},
 'no aaa new-model': {},
 'no ip domain lookup': {},
 'no logging btrace': {},
 'no logging console': {},
 'performance-measurement': {'interface Ethernet2/0': {'delay-measurement': {'advertise-delay 10': {}}},
                             'interface Ethernet2/1': {'delay-measurement': {'advertise-delay 10': {}}},
                             'interface Ethernet3/0': {'delay-measurement': {'advertise-delay 10': {}}},
                             'interface Ethernet3/1': {'delay-measurement': {'advertise-delay 10': {}}},
                             'interface Ethernet5/0': {'delay-measurement': {'advertise-delay 10': {}}},
                             'interface Ethernet5/1': {'delay-measurement': {'advertise-delay 10': {}}}},
 'route-map odn permit 10': {'set extcommunity color 10': {}},
 'route-map pe2 permit 10': {'set local-preference 30': {}},
 'router bgp 65000': {'address-family ipv4': {'redistribute static route-map test': {}},
                      'address-family ipv4 vrf ce1': {'neighbor 100.1.3.1 activate': {},
                                                      'neighbor 100.1.3.1 remote-as 65001': {},
                                                      'neighbor 100.1.3.1 send-community extended': {},
                                                      'neighbor 99.1.3.1 activate': {},
                                                      'neighbor 99.1.3.1 remote-as 65001': {},
                                                      'neighbor 99.1.3.1 send-community extended': {}},
                      'address-family ipv6': {},
                      'address-family ipv6 vrf ce1': {'neighbor 2001::100:1:3:1 activate': {},
                                                      'neighbor 2001::100:1:3:1 remote-as 65001': {},
                                                      'neighbor 2001::100:1:3:1 send-community extended': {},
                                                      'neighbor 2001::99:1:3:1 activate': {},
                                                      'neighbor 2001::99:1:3:1 remote-as 65001': {},
                                                      'neighbor 2001::99:1:3:1 send-community extended': {}},
                      'address-family vpnv4': {'exit-srv6': {},
                                               'neighbor 1.1.1.4 activate': {},
                                               'neighbor 1.1.1.4 next-hop-self': {},
                                               'neighbor 1.1.1.4 route-map pe2 out': {},
                                               'neighbor 1.1.1.4 route-map test in': {},
                                               'neighbor 1.1.1.4 send-community extended': {},
                                               'neighbor 1.1.1.6 activate': {},
                                               'neighbor 1.1.1.6 next-hop-self': {},
                                               'neighbor 1.1.1.6 route-map odn in': {},
                                               'neighbor 1.1.1.6 route-map pe2 out': {},
                                               'neighbor 1.1.1.6 send-community extended': {},
                                               'neighbor 2001::1:1:1:4 activate': {},
                                               'neighbor 2001::1:1:1:4 next-hop-self': {},
                                               'neighbor 2001::1:1:1:4 route-map odn in': {},
                                               'neighbor 2001::1:1:1:4 route-map pe2 out': {},
                                               'neighbor 2001::1:1:1:4 send-community both': {},
                                               'neighbor 2001::1:1:1:6 activate': {},
                                               'neighbor 2001::1:1:1:6 next-hop-self': {},
                                               'neighbor 2001::1:1:1:6 route-map odn in': {},
                                               'neighbor 2001::1:1:1:6 route-map pe2 out': {},
                                               'neighbor 2001::1:1:1:6 send-community both': {},
                                               'segment-routing srv6': {'alloc-mode per-vrf': {},
                                                                        'locator loc1': {}}},
                      'address-family vpnv6': {'exit-srv6': {},
                                               'neighbor 1.1.1.4 activate': {},
                                               'neighbor 1.1.1.4 next-hop-self': {},
                                               'neighbor 1.1.1.4 route-map odn in': {},
                                               'neighbor 1.1.1.4 route-map pe2 out': {},
                                               'neighbor 1.1.1.4 send-community extended': {},
                                               'neighbor 1.1.1.6 activate': {},
                                               'neighbor 1.1.1.6 next-hop-self': {},
                                               'neighbor 1.1.1.6 route-map odn in': {},
                                               'neighbor 1.1.1.6 route-map pe2 out': {},
                                               'neighbor 1.1.1.6 send-community extended': {},
                                               'neighbor 2001::1:1:1:4 activate': {},
                                               'neighbor 2001::1:1:1:4 next-hop-self': {},
                                               'neighbor 2001::1:1:1:4 route-map odn in': {},
                                               'neighbor 2001::1:1:1:4 route-map pe2 out': {},
                                               'neighbor 2001::1:1:1:4 send-community both': {},
                                               'neighbor 2001::1:1:1:6 activate': {},
                                               'neighbor 2001::1:1:1:6 next-hop-self': {},
                                               'neighbor 2001::1:1:1:6 route-map odn in': {},
                                               'neighbor 2001::1:1:1:6 route-map pe2 out': {},
                                               'neighbor 2001::1:1:1:6 send-community both': {},
                                               'segment-routing srv6': {'alloc-mode per-vrf': {},
                                                                        'locator loc1': {}}},
                      'bgp graceful-restart all': {},
                      'bgp log-neighbor-changes': {},
                      'bgp router-id interface Loopback1': {},
                      'exit-address-family': {},
                      'exit-srv6': {},
                      'neighbor 1.1.1.4 remote-as 65000': {},
                      'neighbor 1.1.1.4 update-source Loopback1': {},
                      'neighbor 1.1.1.6 remote-as 65000': {},
                      'neighbor 1.1.1.6 update-source Loopback1': {},
                      'neighbor 2001::1:1:1:4 remote-as 65000': {},
                      'neighbor 2001::1:1:1:4 update-source Loopback1': {},
                      'neighbor 2001::1:1:1:6 remote-as 65000': {},
                      'neighbor 2001::1:1:1:6 update-source Loopback1': {},
                      'no bgp default ipv4-unicast': {},
                      'segment-routing srv6': {'locator loc1': {}}},
 'router isis sr': {'address-family ipv6': {'multi-topology': {},
                                            'router-id Loopback1': {},
                                            'segment-routing srv6': {'locator loc1': {'level-2': {}}}},
                    'distribute link-state': {},
                    'exit-address-family': {},
                    'is-type level-2-only': {},
                    'metric-style wide': {},
                    'mpls traffic-eng level-2': {},
                    'mpls traffic-eng router-id Loopback1': {},
                    'net 49.0000.0000.0003.00': {},
                    'segment-routing mpls': {}},
 'segment-routing mpls': {'connected-prefix-sid-map': {'address-family ipv4': {'1.1.1.3/32 index 3 range 1': {}},
                                                       'exit-address-family': {}},
                          'global-block 16000 23999': {},
                          'local-block 30000 39999': {},
                          'set-attributes': {'address-family ipv4': {'sr-label-preferred': {}},
                                             'exit-address-family': {}}},
 'segment-routing srv6': {'locators': {'locator loc1': {'format usid-f3216': {},
                                                        'prefix FC01:101:3::/48': {}}}},
 'segment-routing traffic-eng': {'on-demand color 10': {'authorize': {},
                                                        'candidate-paths': {'preference 1': {'constraints': {'segments': {'dataplane srv6': {}}},
                                                                                             'dynamic': {}}}}},
 'service compress-config': {},
 'service internal': {},
 'service tcp-keepalives-in': {},
 'service tcp-keepalives-out': {},
 'service timestamps debug datetime msec': {},
 'service timestamps log datetime msec': {},
 'spanning-tree mode rapid-pvst': {},
 'version 17.13': {},
 'vrf definition ce1': {'address-family ipv4': {'route-target export 1:1': {},
                                                'route-target import 1:1': {}},
                        'address-family ipv6': {'route-target export 1:1': {},
                                                'route-target import 1:1': {}},
                        'exit-address-family': {},
                        'rd 3:3': {}}}
        self.assertEqual(result, expected_output)
